/* ISC license. */

/* Generated by: ./configure --prefix=/usr --sysconfdir=/etc --disable-allstatic --disable-static --enable-pkgconfig --enable-shared --with-pkgconfig */

#ifndef EXECLINE_CONFIG_H
#define EXECLINE_CONFIG_H

#define EXECLINE_VERSION "2.9.9.1"
#define EXECLINE_BINPREFIX ""
#define EXECLINE_EXTBINPREFIX ""
#define EXECLINE_EXTLIBEXECPREFIX "/usr/libexec/"
#define EXECLINE_LIBEXECPREFIX "/usr/libexec/"
#define EXECLINE_SYSCONFPREFIX "/etc/"
#define EXECLINE_SHEBANGPREFIX "/usr/bin/"

#undef EXECLINE_PEDANTIC_POSIX

#endif
